cvs_id = "$Id: PollManager.py,v 1.25.2.1 2003/08/16 22:30:10 juri Exp $"

import straw
import time
import gtk
from error import log, logtb
from urllib import quote as urlquote
import traceback
import socket

class PollManager:
    def __init__(self):
        self._lastpoll = -1
        self._poll_frequency = straw.config.poll_frequency
        straw.config.signal_connect(straw.PollFrequencyChangedSignal,
                              self.poll_freq_changed)
        self.pollers = []
        gtk.timeout_add(10000, self.maybe_poll)

    def poll_freq_changed(self, signal):
        self._poll_frequency = signal.sender.poll_frequency

    def maybe_poll(self):
        now = time.time()
        if (self._poll_frequency > 0) and (now - self._lastpoll > self._poll_frequency):
            self._lastpoll = now
            # avoid polling when offline
            if not straw.config.offline:
                self.poll()
        gtk.timeout_add(10000, self.maybe_poll)

    def poll(self, feed = None):
        if straw.config.offline:
            straw.main.show_offline_dialog()
            return None
        straw.main.display_status_message(_("Starting polling"))
        if feed:
            feeds = [feed]
        else:
            feeds = straw.config.feeds
        for f in feeds:
            if f not in self.pollers:
                straw.main.display_status_message(
                    _("Polling %s...") % f.title)
                self.pollers.append(f)
                p = Poller(f, self)
                p.poll()

    def poller_finished(self, feed):
        self.pollers.remove(feed)

class Poller:
    _parser = straw.SummaryParser()
    _parsermp = straw.SummaryParserMP()

    def __init__(self, feed, pm):
        self._feed = feed
        self._pm = pm

    def poll(self):
        url, user, pw = self._feed.get_access_info()
        parsed = None
        if url is None or url == "":
            return None

        headers = {}
        if self._feed.previous_etag is not None:
            headers['If-None-Match'] = self._feed.previous_etag
        try:
            pc = straw.URLFetch.connection_manager.request(
                url, self, headers, user, pw,
                priority=straw.NetworkConstants.PRIORITY_RSS)
            self._feed.router.start_polling()
        except Exception, e:
            self.http_failed(e)

    def http_results(self, status, header, data):
        success = self.update_feed(status, header, data)
        if success:
            straw.main.display_status_message(
                _("Parsing data of %s done.") % self._feed.title)
        else:
            straw.main.display_status_message(
                _("Parsing data of %s failed.") % self._feed.title)
        self._feed.poll_done()
        self._pm.poller_finished(self._feed)

    def update_feed(self, status, header, data):
        straw.main.display_status_message(
            _("Parsing data for feed %s...") % self._feed.title)
        if status[1] == 304:
            self._feed.router.route_no_data()
            return 1
        if status[1] == 410:
            self._feed.router.set_error(_("Cannot found feed.") % status[1])
            return None
        elif status[1] != 200:
            self._feed.router.set_error(_("Polling feed resulted in abnormal status '%s' (code %d)") % (status[2], status[1]))
            return None
        parsed = None
        try:
            parsed = self._parser.parse(data, self._feed)
        except straw.RSSParseException, e:
            parsed = self._parsermp.parse(data, self._feed)
            parsed.invalid_rss = 1
        except Exception, e:
            self._feed.error = _(
                "Unknown exception when parsing RSS: %s") % str(e)
            return None
        self._feed.router.route_all(header, parsed)
        return 1

    def http_failed(self, exception):
        if isinstance(exception, socket.error):
            err = exception.args[1]
        else:
            err = str(exception)
        self._feed.router.set_error(err)
        straw.main.display_status_message(
            _("Polling of feed %s failed") % self._feed.title)
        self._feed.poll_done()
        self._pm.poller_finished(self._feed)

    def http_permanent_redirect(self, location):
        (oldloc, u, p) = self._feed.access_info
        self._feed.access_info = (location, u, p)

class RSSParseException(Exception):
    pass
